#include <stdio.h>
#include <string.h>
#include "../plugin/plugin.h"
#include "test_helpers.h"

// Architecture-specific macros
#ifdef ARCHITECTURE_x86
    #define ARCHITECTURE_NAME "x86"
    // x86-specific opcodes and registers
    #define JMP_REGISTER_OPCODE 0xE0  // JMP EAX
    #define CALL_REGISTER_OPCODE 0xD0 // CALL EAX
#else
    #define ARCHITECTURE_NAME "x64"
    // x64-specific opcodes and registers
    #define JMP_REGISTER_OPCODE 0xE0  // JMP RAX
    #define CALL_REGISTER_OPCODE 0xD0 // CALL RAX
#endif

// Функция для тестирования инструкций JMP, CALL, RET
bool test_jmp_call_ret() {
    // Здесь реализуем тест для инструкций JMP, CALL, RET
    printf("Testing JMP, CALL, RET instructions on %s architecture...\n", ARCHITECTURE_NAME);

    // Пример тестовых данных
    bool jmp_test_passed = true;
    bool call_test_passed = true;
    bool ret_test_passed = true;

    // Тестирование JMP инструкций
    printf("Testing JMP instructions...\n");

    // Создаем тестовые данные для JMP
    unsigned char jmp_near[] = {0xE9, 0x10, 0x00, 0x00, 0x00}; // JMP rel32 (E9 cd)
    unsigned char jmp_short[] = {0xEB, 0x10}; // JMP rel8 (EB cb)
    unsigned char jmp_register[] = {0xFF, JMP_REGISTER_OPCODE}; // JMP EAX/RAX (FF /4)
    unsigned char jmp_memory[] = {0xFF, 0x25, 0x00, 0x00, 0x00, 0x00}; // JMP [mem] (FF /5)

    // Проверяем распознавание JMP инструкций
    if (!IsJumpInstruction(jmp_near, sizeof(jmp_near))) {
        printf("Failed to recognize JMP near instruction\n");
        jmp_test_passed = false;
    }

    if (!IsJumpInstruction(jmp_short, sizeof(jmp_short))) {
        printf("Failed to recognize JMP short instruction\n");
        jmp_test_passed = false;
    }

    if (!IsJumpInstruction(jmp_register, sizeof(jmp_register))) {
        printf("Failed to recognize JMP register instruction\n");
        jmp_test_passed = false;
    }

    if (!IsJumpInstruction(jmp_memory, sizeof(jmp_memory))) {
        printf("Failed to recognize JMP memory instruction\n");
        jmp_test_passed = false;
    }

    // Тестирование CALL инструкций
    printf("Testing CALL instructions...\n");

    // Создаем тестовые данные для CALL
    unsigned char call_near[] = {0xE8, 0x10, 0x00, 0x00, 0x00}; // CALL rel32 (E8 cd)
    unsigned char call_register[] = {0xFF, CALL_REGISTER_OPCODE}; // CALL EAX/RAX (FF /2)
    unsigned char call_memory[] = {0xFF, 0x15, 0x00, 0x00, 0x00, 0x00}; // CALL [mem] (FF /3)

    // Проверяем распознавание CALL инструкций и логируем адреса
    printf("CALL instruction address logging:\n");

    // Симулируем адреса в памяти для тестирования
    uintptr_t call_near_addr = 0x401000;
    uintptr_t call_register_addr = 0x402000;
    uintptr_t call_memory_addr = 0x403000;

    // Проверяем и логируем CALL near
    if (IsCallInstruction(call_near, sizeof(call_near))) {
        // Вычисляем целевой адрес для CALL near (относительный переход)
        int32_t offset = *((int32_t*)&call_near[1]);
        uintptr_t target_addr = call_near_addr + sizeof(call_near) + offset;

        printf("CALL near at 0x%p, target: 0x%p, offset: %d\n",
               (void*)call_near_addr, (void*)target_addr, offset);
    } else {
        printf("Failed to recognize CALL near instruction\n");
        call_test_passed = false;
    }

    // Проверяем и логируем CALL register
    if (IsCallInstruction(call_register, sizeof(call_register))) {
        printf("CALL register at 0x%p, target: [%cAX]\n",
               (void*)call_register_addr,
               ARCHITECTURE_NAME[1]); // 'E' for x86, 'R' for x64
    } else {
        printf("Failed to recognize CALL register instruction\n");
        call_test_passed = false;
    }

    // Проверяем и логируем CALL memory
    if (IsCallInstruction(call_memory, sizeof(call_memory))) {
        // Для CALL memory нужно прочитать адрес из памяти
        // В реальном коде это был бы указатель, но для теста используем фиктивное значение
        uintptr_t memory_target = 0x500000;
        printf("CALL memory at 0x%p, target: [0x%p] -> 0x%p\n",
               (void*)call_memory_addr,
               (void*)(call_memory_addr + 2), // Адрес, откуда читается указатель
               (void*)memory_target);         // Целевой адрес (фиктивный)
    } else {
        printf("Failed to recognize CALL memory instruction\n");
        call_test_passed = false;
    }

    // Тестирование RET инструкций
    printf("Testing RET instructions...\n");

    // Создаем тестовые данные для RET
    unsigned char ret_simple[] = {0xC3}; // RET (C3)
    unsigned char ret_imm[] = {0xC2, 0x04, 0x00}; // RET imm16 (C2 iw)
    unsigned char ret_far[] = {0xCB}; // RET far (CB)
    unsigned char ret_far_imm[] = {0xCA, 0x04, 0x00}; // RET far imm16 (CA iw)

    // Проверяем распознавание RET инструкций
    if (!IsRetInstruction(ret_simple, sizeof(ret_simple))) {
        printf("Failed to recognize simple RET instruction\n");
        ret_test_passed = false;
    }

    if (!IsRetInstruction(ret_imm, sizeof(ret_imm))) {
        printf("Failed to recognize RET imm16 instruction\n");
        ret_test_passed = false;
    }

    if (!IsRetInstruction(ret_far, sizeof(ret_far))) {
        printf("Failed to recognize RET far instruction\n");
        ret_test_passed = false;
    }

    if (!IsRetInstruction(ret_far_imm, sizeof(ret_far_imm))) {
        printf("Failed to recognize RET far imm16 instruction\n");
        ret_test_passed = false;
    }

    // Проверяем, что другие инструкции не распознаются как JMP/CALL/RET
    unsigned char not_control_flow[] = {0x90}; // NOP

    if (IsJumpInstruction(not_control_flow, sizeof(not_control_flow))) {
        printf("Incorrectly recognized NOP as JMP instruction\n");
        jmp_test_passed = false;
    }

    if (IsCallInstruction(not_control_flow, sizeof(not_control_flow))) {
        printf("Incorrectly recognized NOP as CALL instruction\n");
        call_test_passed = false;
    }

    if (IsRetInstruction(not_control_flow, sizeof(not_control_flow))) {
        printf("Incorrectly recognized NOP as RET instruction\n");
        ret_test_passed = false;
    }

    // Выводим результаты тестов
    printf("JMP test %s\n", jmp_test_passed ? "PASSED" : "FAILED");
    printf("CALL test %s\n", call_test_passed ? "PASSED" : "FAILED");
    printf("RET test %s\n", ret_test_passed ? "PASSED" : "FAILED");

    // Возвращаем true, если все тесты прошли успешно
    return jmp_test_passed && call_test_passed && ret_test_passed;
}

// Структура для тестов
typedef struct {
    const char* name;
    bool (*test_function)();
} TestCase;

// Основная функция для запуска тестов
int main() {
    printf("Running tests on %s architecture\n", ARCHITECTURE_NAME);

    // Массив тестов
    TestCase tests[] = {
        {"Test JMP, CALL, RET instructions", test_jmp_call_ret}
        // Здесь можно добавить другие тесты
    };

    int test_count = sizeof(tests) / sizeof(tests[0]);
    int passed = 0;

    printf("Running %d tests...\n\n", test_count);

    for (int i = 0; i < test_count; i++) {
        printf("Test %d: %s\n", i + 1, tests[i].name);
        bool result = tests[i].test_function();
        if (result) {
            printf("PASSED\n");
            passed++;
        } else {
            printf("FAILED\n");
        }
        printf("\n");
    }

    printf("Test results: %d/%d passed\n", passed, test_count);

    return passed == test_count ? 0 : 1;
}
